"""
PyXLL Examples: Real time data

As well as returning static values from functions, PyXLL functions
can return special 'RTD' instances that can notify Excel of
updates to their value.

This could be used for any real time data feed, such as live
prices or the status of a service.
"""
from pyxll import RTD, xl_func, xl_app
from datetime import datetime
import threading
import logging
import time
import random
import json
import urllib.request, urllib.error, urllib.parse

_log = logging.getLogger(__name__)

class RandomRTD(RTD):

    def __init__(self):
        initial_value = 0
        super(RandomRTD, self).__init__(value=initial_value)
        self.__running = True
        self.__thread = threading.Thread(target=self.__thread_func)
        self.__thread.start()

    def connect(self):
        # Called when Excel connects to this RTD instance, which occurs
        # shortly after an Excel function has returned an RTD object.
        _log.info("RandomRTD Connected")

    def disconnect(self):
        # Called when Excel no longer needs the RTD instance. This is
        # usually because there are no longer any cells that need it
        # or because Excel is shutting down.
        self.__running = False
        _log.info("RandomRTD Disconnected")

    def __thread_func(self):
        while self.__running:
            # Setting 'value' on an RTD instance triggers an update in Excel
            try:
                url = "https://www.random.org/integers/?num=1&min=1&max=99&col=1&base=10&format=plain&rnd=new"
                data = urllib.request.urlopen(url).read()
                result = data.decode()
            except Exception as e:
                result = e
            self.value = int(result)
            time.sleep(random.randrange(2,10))

@xl_func(" : rtd")
def random_rtd():
    return RandomRTD()
